/*
 * Copyright 2011 Apaq.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package dk.apaq.filter.lucene;

import dk.apaq.filter.Filter;
import dk.apaq.filter.core.AndFilter;
import dk.apaq.filter.core.CompareFilter;
import dk.apaq.filter.core.LikeFilter;
import dk.apaq.filter.core.NotFilter;
import dk.apaq.filter.core.OrFilter;


/**
 *
 * @author michaelzachariassenkrog
 */
public class FiltrationLuceneTranslator {

    private static final FiltrationLuceneTranslator SINGLETON = new FiltrationLuceneTranslator();
    
    public static FiltrationLuceneTranslator getTranslator() {
        return SINGLETON;
    }

    public String translate(Filter filter) {
        if(filter instanceof LikeFilter) {
            LikeFilter cf = (LikeFilter)filter;
            if(cf.getValue() == null) {
                return null;
            }

            String searchString = cf.getValue().trim();
            searchString = searchString.replace(" *", "");
            
            String[] valueArray = searchString.split(" ");
            StringBuilder sb = new StringBuilder();
            
            for(String value:valueArray) {
                value = value.trim();
                if(value.startsWith("-") || value.startsWith("+")) {
                    continue;
                }

                if(sb.length()>0) {
                    sb.append(" AND ");
                }

                if(value.endsWith("*")) {
                    value = value.substring(0, value.length()-1);
                }

                sb.append("(");
                sb.append(cf.getPropertyId());
                sb.append(":");
                sb.append(escapeSearchTerm(value));
                if(cf.getPrecision() < 1) {
                    sb.append("~");
                    sb.append(cf.getPrecision());
                }
                sb.append(" OR ");
                sb.append(cf.getPropertyId());
                sb.append(":");
                sb.append(escapeSearchTerm(value));
                sb.append("*");
                sb.append(")");
            }

            sb.insert(0, "(");
            sb.append(")");
            return sb.toString();
        }

        if(filter instanceof CompareFilter) {
            CompareFilter cf = (CompareFilter)filter;
            switch(cf.getCompareType()) {
                case Equals:
                    return cf.getPropertyId() + ":" +cf.getValue();

                    //TODO
                case GreaterOrEqual:
                    return cf.getPropertyId() + ":[" + cf.getValue() + " TO " + Long.MAX_VALUE + "]";
                case GreaterThan:
                    return cf.getPropertyId() + ":{" + cf.getValue() + " TO " + Long.MAX_VALUE + "}";
                case LessOrEqual:
                    return cf.getPropertyId() + ":[0 TO " + cf.getValue() + "]";
                case LessThan:
                    return cf.getPropertyId() + "{0 TO " + cf.getValue() + "}";
            }
        }

        if(filter instanceof AndFilter) {
            AndFilter af = (AndFilter) filter;
            StringBuilder sb = new StringBuilder();

            //sb.append("(");
            for(int i=0;i<af.getFilters().size();i++) {
                if (i>0) {
                   sb.append(" AND ");
                }
                sb.append(translate(af.getFilters().get(i)));
            }
            //sb.append(")");
            return sb.toString();
        }

        if(filter instanceof OrFilter) {
            OrFilter af = (OrFilter) filter;
            StringBuilder sb = new StringBuilder();

            //sb.append("(");
            for(int i=0;i<af.getFilters().size();i++) {
                if (i>0) {
                   sb.append(" OR ");
                }
                sb.append(translate(af.getFilters().get(i)));
            }
            //sb.append(")");
            return sb.toString();
        }

        if(filter instanceof NotFilter) {
            NotFilter af = (NotFilter) filter;
            return " NOT " + translate(af.getFilterElement());
        }

        return null;
    }

    private static String escapeSearchTerm(String term) {
        return term.replaceAll("\"", "\\\"");
    }
}
